//! Authentication
use super::*;

pub struct Sha256Hash(pub sha256::Sha256Hash);
type RsaSignature = rsa::Signature;

impl str::FromStr for Sha256Hash
{
    type Err = ();
    fn from_str(s: &str) -> Result<Self, Self::Err> {
	todo!() //read encoded base64(?)/hex into `Signature`
    }
}

#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
pub struct AuthRequest
{
    id: Uuid,
    
    sign_this: [u8; 32],
    salt: [u8; 16],
    passwd_is_allowed: bool,
    ttl_ms: u64
}

impl AuthRequest
{
    /// Create a new auth request
    pub fn new(cfg: &settings::Settings) -> Self
    {
	let mut empty = Self {
	    id: Uuid::new_v4(),
	    sign_this: [0; 32],
	    salt: [0;16],
	    passwd_is_allowed: cfg.allow_passwd_auth,
	    ttl_ms: cfg.auth_req_ttl_millis.jitter(),
	};
	getrandom(&mut empty.sign_this[..]).expect("fatal rng");
	getrandom(&mut empty.salt[..]).expect("fatal rng");
	empty
    }
}

pub async fn auth_req(who: source::IpAddr, state: Arc<State>) -> Result<AuthRequest, Infallible>
{
    let req = AuthRequest::new(state.cfg());
    trace!("{:?} auth req", who);
    
    // TODO: Add `req` into `state` auth hashmap (`req.id` is key) for verification.
    // TODO: Use `DelayQueue` to remove `req.id` from the hashmap after `ttl` expires.
    Ok(req)
}

pub async fn auth_key(who: source::IpAddr, state: Arc<State>, req_id: Uuid, num: usize, body: Bytes) -> Result<(), Infallible>
{
    trace!("{:?} auth resp key <{}>:{}", who, req_id, num);
    
    Ok(())
}

pub async fn auth_pass(who: source::IpAddr, state: Arc<State>, req_id: Uuid, passhash: sha256::Sha256Hash) -> Result<(), Infallible>
{
    trace!("{:?} auth resp pass <{}>: \"{}\"", who, req_id, passhash);
    
    Ok(())
}
