//! Settings for web server
//!
//! Usually derived from config
use super::*;

const DEFAULT_MAX_BODY_LEN_ARESP: u64 = 1024 * 4;  // 4KB
const DEFAULT_MAX_BODY_LEN: u64 = 1024 * 1024 * 4; // 4MB

#[derive(Debug, Clone, PartialEq, Eq, Hash)]
pub struct Settings
{
    /// First is max body len for auth responses, 2nd is for data.
    pub max_body_len: (u64, u64),
    pub trust_x_forwarded_for: bool,

    pub allow_passwd_auth: bool,
    pub max_key_sigs_per_auth_response: usize,

    /// How long is an auth request ID is valid.
    ///
    /// A random value between these two bounds is selected
    pub auth_req_ttl_millis: (u64, u64),
    /// How long is an authentication token valid for an action.
    ///
    /// A random value between these two bounds is selected
    pub auth_token_ttl_millis: (u64, u64),
}

impl Default for Settings
{
    #[inline]
    fn default() -> Self
    {
	Self {
	    max_body_len: (DEFAULT_MAX_BODY_LEN_ARESP, DEFAULT_MAX_BODY_LEN),
	    trust_x_forwarded_for: false,

	    allow_passwd_auth: true,
	    max_key_sigs_per_auth_response: 16,
	    
	    auth_req_ttl_millis: (4000, 6000),        //4s - 6s
	    auth_token_ttl_millis: (1500, 2500).map(|x| x * 60), // 1.5m - 2.5m
	}
    }
}

